﻿using CarRepairShopContracts.BindingModels;
using CarRepairShopContracts.SearchModels;
using CarRepairShopContracts.StoragesContracts;
using CarRepairShopContracts.ViewModels;
using CarRepairShopDatabaseImplement.Models;
using CarRepairShopDataModels.Enums;
using DocumentFormat.OpenXml.InkML;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace CarRepairShopDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new CarRepairShopDatabase();
            return context.Orders.Include(x=>x.Repair).Include(x => x.Client).Include(x=>x.Implementer).Select(x => x.GetViewModel).ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.ClientId.HasValue && !model.Status.HasValue)
            {
                return new();
            }
            using var context = new CarRepairShopDatabase();

            return context.Orders
                .Include(x => x.Repair)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Where(x => x.Id == model.Id || x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo || x.ClientId == model.ClientId || model.Status.Equals(x.Status))
                .Select(x => x.GetViewModel).ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if(!model.Id.HasValue)
            {
                return null;
            }
            using var context = new CarRepairShopDatabase();
            return context.Orders.Include(x=>x.Repair).Include(x => x.Client).Include(x=>x.Implementer).FirstOrDefault(x => x.Id == model.Id || (x.ImplementerId == model.ImplementerId && x.Status == model.Status))?.GetViewModel;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if(newOrder == null)
            {
                return null;
            }
            using var context = new CarRepairShopDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders.Include(x=>x.Repair).Include(x => x.Client).Include(x=>x.Implementer).FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new CarRepairShopDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if(order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders.Include(x=>x.Repair).Include(x => x.Client).Include(x=>x.Implementer).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new CarRepairShopDatabase();
            var element = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if(element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
