﻿using CarRepairShopContracts.BindingModels;
using CarRepairShopContracts.ViewModels;
using CarRepairShopDataModels.Enums;
using CarRepairShopDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace CarRepairShopFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int RepairId { get; private set; }
        public int ClientId { get; private set; }
        public int? ImplementerId { get; set; }
        public int Count { get; private set; }
        public double Sum {  get; private set; }
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }
        public virtual Repair Repair { get; set; }

        public virtual Client Client { get; set; }

        public static Order? Create(OrderBindingModel model)
        {
            if(model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                RepairId = model.RepairId,
                ClientId = model.ClientId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                RepairId = Convert.ToInt32(element.Element("RepairId")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = Enum.Parse<OrderStatus>(element.Element("Status")!.Value),
                DateCreate = DateTime.ParseExact(element.Element("DateCreate")!.Value, "G", null),
                DateImplement = !string.IsNullOrEmpty(element.Element("DateImplement")!.Value) ? DateTime.ParseExact(element.Element("DateImplement")!.Value, "G", null) : null
            };
        }
        public void Update(OrderBindingModel model)
        {
            if(model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            RepairId = RepairId,
            ClientId = ClientId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            RepairName = Repair.RepairName,
            ClientFIO = Client.ClientFIO
        };
        public XElement GetXElement => new("Order",
                        new XAttribute("Id", Id),
                        new XElement("RepairId", RepairId),
                        new XElement("ClientId", ClientId),
                        new XElement("Count", Count.ToString()),
                        new XElement("Sum", Sum.ToString()),
                        new XElement("Status", Status.ToString()),
                        new XElement("DateCreate", DateCreate.ToString()),
                        new XElement("DateImplement", DateImplement.ToString()));
    }
}
