﻿using CarRepairShopContracts.BindingModels;
using CarRepairShopContracts.SearchModels;
using CarRepairShopContracts.StoragesContracts;
using CarRepairShopContracts.ViewModels;
using CarRepairShopListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CarRepairShopListImplement.Implements
{
    public class RepairStorage : IRepairStorage
    {
        private readonly DataListSingleton _source;
        public RepairStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public List<RepairViewModel> GetFullList()
        {
            var result = new List<RepairViewModel>();
            foreach(var Repair in _source.Repairs)
            {
                result.Add(Repair.GetViewModel);
            }
            return result;
        }
        public List<RepairViewModel> GetFilteredList(RepairSearchModel model)
        {
            var result = new List<RepairViewModel>();
            if(string.IsNullOrEmpty(model.RepairName))
            {
                return result;
            }
            foreach(var Repair in _source.Repairs)
            {
                if(Repair.RepairName.Contains(model.RepairName))
                {
                    result.Add(Repair.GetViewModel);
                }
            }
            return result;
        }
        public RepairViewModel? GetElement(RepairSearchModel model)
        {
            if(string.IsNullOrEmpty(model.RepairName) && !model.Id.HasValue)
            {
                return null;
            }
            foreach(var Repair in _source.Repairs)
            {
                if((!string.IsNullOrEmpty(model.RepairName) && model.RepairName == Repair.RepairName) || (model.Id.HasValue && Repair.Id == model.Id))
                {
                    return Repair.GetViewModel;
                }
            }
            return null;
        }
        public RepairViewModel? Insert(RepairBindingModel model)
        {
            model.Id = 1;
            foreach(var Repair in _source.Repairs)
            {
                if(model.Id <= Repair.Id)
                {
                    model.Id = Repair.Id + 1;
                }
            }
            var newRepair = Repair.Create(model);
            if(newRepair == null)
            {
                return null;
            }
            _source.Repairs.Add(newRepair);
            return newRepair.GetViewModel;
        }
        public RepairViewModel? Update(RepairBindingModel model)
        {
            foreach(var Repair in _source.Repairs)
            {
                if(Repair.Id == model.Id)
                {
                    Repair.Update(model);
                    return Repair.GetViewModel;
                }
            }
            return null;
        }
        public RepairViewModel? Delete(RepairBindingModel model)
        {
            for(int i = 0; i< _source.Repairs.Count; i++)
            {
                if (_source.Repairs[i].Id == model.Id)
                {
                    var element = _source.Repairs[i];
                    _source.Repairs.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
